﻿using System;
using System.Threading;
using System.IO.Ports;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;

namespace NetQuadCopter
{
    public class Program
    {
        // Variable para saber si el botón está pulsado o no
        private static bool pressed = false;

        // Variable para controlar el LED externo (salida analógica simulada mediante PWM)
        private static PWM externalLED = null;

        // Variable para controlar el LED interno (salida digital)
        private static OutputPort internalLED = null;

        public static void Main()
        {
            // Inicializo el LED externo
            externalLED = new SecretLabs.NETMF.Hardware.PWM(Pins.GPIO_PIN_D5);

            // Inicializo el LED interno de la placa
            internalLED = new OutputPort(Pins.ONBOARD_LED, false);

            // Inicializo el pulsador de la placa y me suscribo al evento de pulsación
            InterruptPort button = new InterruptPort(Pins.ONBOARD_SW1, false, Port.ResistorMode.Disabled, Port.InterruptMode.InterruptEdgeBoth);
            button.OnInterrupt += new NativeEventHandler(port_OnInterrupt);

            // Creo y lanzo un hilo que va a gestionar el LED interno de la placa
            ThreadStart tStart = new ThreadStart(InternalLEDManager);
            Thread thead = new Thread(tStart);
            thead.Start();

            while (true)
            {
                // Aumentamos desde un 0% a un 100% la potencia (0v a 3.3v)
                for (byte i = 0; i <= 100; i++)
                {
                    externalLED.SetDutyCycle(i);
                    // Después de cada cambio hacemos una pausa para que no sea demasiado rápido
                    Thread.Sleep(10);
                }

                // Disminuimos desde un 100% a un 0% la potencia (3.3v a 0v)
                for (byte i = 100; i > 0; i--)
                {
                    externalLED.SetDutyCycle(i);
                    // Después de cada cambio hacemos una pausa para que no sea demasiado rápido
                    Thread.Sleep(10);
                }
            }
        }

        /// <summary>
        /// Método que se encarga de hacer parpadear el LED interno de la placa
        /// </summary>
        private static void InternalLEDManager()
        {
            while (true)
            {
                // Si está presionado se espera para evitar sobre-ocupar la CPU. Sino lo hacemos parpadear
                if (!pressed)
                {
                    // Encendemos el LED
                    internalLED.Write(true);
                    // Esperamos un poco
                    Thread.Sleep(200);
                    // Apagamos el LED
                    internalLED.Write(false);
                    // Esperamos un poco
                    Thread.Sleep(100);
                }
                else
                    Thread.Sleep(100);
            }
        }

        /// <summary>
        /// Evento que se lanza si se pulsa el botón
        /// </summary>
        /// <param name="data1"></param>
        /// <param name="data2">0 si está pulsado y 1 si no lo está</param>
        /// <param name="time"></param>
        static void port_OnInterrupt(uint data1, uint data2, DateTime time)
        {
            pressed = (data2 == 0);
        }
    }
}
